#pragma once
#include "ns3/config-store-module.h"
#include "ns3/core-module.h"
#include "ns3/energy-module.h"


using namespace ns3;


DeviceEnergyModelContainer deviceModels;
EnergySourceContainer sources;



void RemainingEnergy(double oldValue, double remainingEnergy)
{
	std::cout << Simulator::Now().GetSeconds()
		<< "s Current remaining energy = " << remainingEnergy << "J" << std::endl;
}


void TotalEnergy(double oldValue, double totalEnergy)
{
	std::cout << Simulator::Now().GetSeconds()
		<< "s Total energy consumed by radio = " << totalEnergy << "J" << std::endl;
}


void SetEnergySources(NodeContainer c,float initial_energy = 0.1) {
	/* energy source */
	BasicEnergySourceHelper basicSourceHelper;
	// configure energy source
	basicSourceHelper.Set("BasicEnergySourceInitialEnergyJ", DoubleValue(initial_energy));
	// install source
	sources = basicSourceHelper.Install(c);
	
}
void SetEnergyModel(NodeContainer c,NetDeviceContainer devices,float TxCurrent= 0.0174) {
	/* device energy model */
	WifiRadioEnergyModelHelper radioEnergyHelper;
	// configure radio energy model
	radioEnergyHelper.Set("TxCurrentA", DoubleValue(TxCurrent));
	// install device model
	deviceModels = radioEnergyHelper.Install(devices, sources);

}
void Track_Node_Energy(int nid) {
	// energy source
	Ptr<BasicEnergySource> basicSourcePtr = DynamicCast<BasicEnergySource>(sources.Get(nid));
	basicSourcePtr->TraceConnectWithoutContext("RemainingEnergy", MakeCallback(&RemainingEnergy));
	// device energy model
	Ptr<DeviceEnergyModel> basicRadioModelPtr =
		basicSourcePtr->FindDeviceEnergyModels("ns3::WifiRadioEnergyModel").Get(nid);
	NS_ASSERT(basicRadioModelPtr);
	basicRadioModelPtr->TraceConnectWithoutContext("TotalEnergyConsumption",
		MakeCallback(&TotalEnergy));
}
void Print_Energy_Levels() {
	for (DeviceEnergyModelContainer::Iterator iter = deviceModels.Begin();
		iter != deviceModels.End();
		iter++)
	{
		double energyConsumed = (*iter)->GetTotalEnergyConsumption();
		printf(("End of simulation ("
			+ std::to_string(Simulator::Now().GetSeconds())
			+ "s) Total energy consumed by radio = " + std::to_string(energyConsumed) + "J").c_str());
		
	}


}