#pragma once
#include "ns3/config-store-module.h"
#include "ns3/core-module.h"
#include "ns3/internet-module.h"
#include "ns3/mobility-module.h"
#include<vector>
#include<map>
using namespace ns3;
std::vector<MobilityHelper*> mobility_models;


/*Sample of providing Positions
Set_Constant_Position_Mobility(c,{{x1,y1},{x2,y2},{x3,y3}....{xn,yn}});

*/
void Set_Constant_Position_Mobility(NodeContainer c,std::vector<std::pair<int,int>>Positions){
	/** mobility **/
	MobilityHelper* mobility=new MobilityHelper;
	Ptr<ListPositionAllocator> positionAlloc = CreateObject<ListPositionAllocator>();
	for (int node = 0; node < Positions.size();node++) {
		positionAlloc->Add(Vector(Positions[node].first, Positions[node].second, 0.0));
	}
	mobility->SetPositionAllocator(positionAlloc);
	mobility->SetMobilityModel("ns3::ConstantPositionMobilityModel");
	mobility->Install(c);
	mobility_models.push_back(mobility);
}

void Set_Random_Walk_Mobility(NodeContainer c) {
	/** mobility **/
	
	MobilityHelper* mobility = new MobilityHelper();
	
	mobility->SetPositionAllocator("ns3::GridPositionAllocator",
		                                  "MinX", DoubleValue(5.0),
		                                  "MinY", DoubleValue(5.0),
		                                  "DeltaX", DoubleValue(15.0),
		                                  "DeltaY", DoubleValue(15.0),
		                                  "GridWidth", UintegerValue(3),
		                                  "LayoutType", StringValue("RowFirst"));
	
	mobility->SetMobilityModel("ns3::RandomWalk2dMobilityModel",
		                              "Mode", StringValue("Time"),
		                              "Time", StringValue("2s"),
		                              "Speed", StringValue("ns3::ConstantRandomVariable[Constant=5.0]"),
		                              "Bounds", RectangleValue(Rectangle(0.0, 60.0, 0.0, 60.0)));

	mobility->Install(c);
	mobility_models.push_back(mobility);
}