#pragma once
#include "ns3/config-store-module.h"
#include "ns3/core-module.h"
#include "ns3/energy-module.h"
#include "ns3/internet-module.h"
#include "ns3/mobility-module.h"
#include "ns3/network-module.h"
#include "ns3/wifi-radio-energy-model-helper.h"
#include "ns3/yans-wifi-helper.h"


#include <fstream>
#include <iostream>
#include <string>
#include <vector>


using namespace ns3;

void Wifi_Simulation() {
	NS_LOG_COMPONENT_DEFINE("Energy_Mobility_Example");
	LogComponentEnable("Energy_Mobility_Example",
		LogLevel(LOG_PREFIX_TIME | LOG_PREFIX_NODE | LOG_LEVEL_INFO));

	std::string phyMode("DsssRate1Mbps");
	double Prss = -80;          // dBm

	// disable fragmentation for frames below 2200 bytes
	Config::SetDefault("ns3::WifiRemoteStationManager::FragmentationThreshold",
		StringValue("2200"));
	// turn off RTS/CTS for frames below 2200 bytes
	Config::SetDefault("ns3::WifiRemoteStationManager::RtsCtsThreshold", StringValue("2200"));
	// Fix non-unicast data rate to be the same as that of unicast
	Config::SetDefault("ns3::WifiRemoteStationManager::NonUnicastMode", StringValue(phyMode));


	Ptr<Node> a = CreateObject<Node>();
	Names::Add("RouterA", a);
	Ptr<Node> b = CreateObject<Node>();
	Names::Add("RouterB", b);
	Ptr<Node> c = CreateObject<Node>();
	Names::Add("RouterC", c);
	Ptr<Node> d = CreateObject<Node>();
	Names::Add("RouterD", d);



	NodeContainer networkNodes(a, b, c, d);

	// The below set of helpers will help us to put together the wifi NICs we want
	WifiHelper wifi;
	wifi.SetStandard(WIFI_STANDARD_80211b);

	/** Wifi PHY **/
	/***************************************************************************/
	YansWifiPhyHelper wifiPhy;

	/** wifi channel **/
	YansWifiChannelHelper wifiChannel;
	wifiChannel.SetPropagationDelay("ns3::ConstantSpeedPropagationDelayModel");
	wifiChannel.AddPropagationLoss("ns3::FriisPropagationLossModel");

	// create wifi channel
	Ptr<YansWifiChannel> wifiChannelPtr = wifiChannel.Create();
	wifiPhy.SetChannel(wifiChannelPtr);

	/** MAC layer **/
	// Add a MAC and disable rate control
	WifiMacHelper wifiMac;
	wifi.SetRemoteStationManager("ns3::ConstantRateWifiManager",
		"DataMode",
		StringValue(phyMode),
		"ControlMode",
		StringValue(phyMode));
	// Set it to ad-hoc mode
	wifiMac.SetType("ns3::AdhocWifiMac");

	/** install PHY + MAC **/
	NetDeviceContainer devices = wifi.Install(wifiPhy, wifiMac, networkNodes);

	
	/** Internet stack **/
	InternetStackHelper internet;
	internet.Install(networkNodes);
	
	Ipv4AddressHelper ipv4;
	NS_LOG_INFO("Assign IP Addresses.");
	ipv4.SetBase("10.1.1.0", "255.255.255.0");
	Ipv4InterfaceContainer i = ipv4.Assign(devices);


	/***************************************************************************/

	/** simulation setup **/


	Simulator::Stop(Seconds(100.0));
	Simulator::Run();
	Simulator::Destroy();
	std::cout << "Simulation Ended";

}

